###############################################################################
# UsersTodayAdmin.pm (Users Today Admin Script)                               #
###############################################################################
# YaBB: Yet another Bulletin Board                                            #
# Open-Source Community Software for Webmasters                               #
# Version:        YaBB 2.7.00                                                 #
# Distributed by: http://www.yabbforum.com                                    #
# =========================================================================== #
# Copyright (c) 2000-2017 YaBB (www.yabbforum.com) - All Rights Reserved.     #
# Software by:  The YaBB Development Team                                     #
#               with assistance from the YaBB community.                      #
###############################################################################
use strict;
use warnings;
no warnings qw(uninitialized);
use Time::Local;
use CGI::Carp qw(fatalsToBrowser);
our $VERSION = '2.7.00';

our $userstodayadminpmver = 'YaBB 2.7.00 $Revision: 1.4 $';
our ($action);
$action ||= q{};
if ( $action eq 'detailedversion' ) { return 1; }

our (
    $action_area, $adminurl,      $vardir,  $year,
    $yymain,      $yysetlocation, $yytitle, %admin_img,
    %admin_txt,   %croak,         %FORM,    %uttxt,
);

load_language('Mods/UsersToday');
load_language('Admin');

our (
    $baddatee,              $baddates,               $clean_log,
    $clear_log_method,      $count_log,              $date_style,
    $dateformat_guest,      $dateformat_guest_stats, $dateformat_log,
    $dateformat_user_stats, $datelogopt21,           $datelogopt22,
    $datestyle,             $days_show,              $delete_old,
    $ed,                    $em,                     $ey2,
    $ey4,                   $fd,                     $fm,
    $fy,                    $gday,                   $guest_ip,
    $guest_view,            $ld,                     $link_guest,
    $lm,                    $log_only,               $logde,
    $logds,                 $logme,                  $logms,
    $logopt,                $logye,                  $logys,
    $ly,                    $mempercent,             $plo2,
    $plogopt,               $plogopt2,               $recover_log,
    $run,                   $sd,                     $show_guest,
    $show_guest_today,      $show_max_stats,         $show_utlog,
    $sm,                    $sy2,                    $sy4,
    $totguest,              $totuser,                $user_today_hidden_show,
    $utday,                 $utdow,                  $utmgday,
    $utmgmo,                $utmgyr,                 $utmonth,
    $utmostguest,           $utmostuser,             $utmuday,
    $utmumo,                $utmuyr,                 $utyear,
    $view_ips,
);

if ( !-e "$vardir/Mods/utoday/usrtodayconfig.txt" ) {
    $fm       = 0;
    $fd       = 0;
    $fy       = 0;
    $lm       = 0;
    $ld       = 0;
    $ly       = 0;
    $sm       = 0;
    $sd       = 0;
    $sy2      = 0;
    $sy4      = 0;
    $em       = 0;
    $ed       = 0;
    $ey2      = 0;
    $ey4      = 0;
    $baddates = 0;
    $baddatee = 0;
    $run      = -1;
    userstoday_set2();
}

require "$vardir/Mods/utoday/usrtodayconfig.txt";

sub userstoday_set {
    check_dates();

    $yymain .= qq~
<form action="$adminurl?action=userstoday2" method="post">
    <div class="bordercolor rightboxdiv">
        <table class="border-space pad-cell" style="margin-bottom: .5em;">
            <tr>
                <td class="titlebg">$admin_img{'prefimg'} <b>$uttxt{'100'}</b></td>
            </tr><tr>
                <td class="catbg"><span class="small">$uttxt{'101'} <i>$uttxt{'102'}</i></span></td>
            </tr>
        </table>
    </div>
    <div class="bordercolor rightboxdiv">
        <table class="border-space pad-cell" style="margin-bottom: .5em;">
            <colgroup>
                <col style="width:50%" span="2" />
            </colgroup>
            <tr>
                <td class="titlebg" colspan="2">$admin_img{'prefimg'} <b>$uttxt{'40'}</b></td>
            </tr><tr>
                <td class="windowbg2">$uttxt{'4b'}</td>
                <td class="windowbg2"><input type="checkbox" name="log_only"${ischecked($log_only)} /></td>
            </tr>~;
    if ( !$log_only ) {
        $yymain .= qq~<tr>
                <td class="windowbg2">$uttxt{'5'}</td>
                <td class="windowbg2"><input type="checkbox" name="guest_view"${ischecked($guest_view)} /></td>
            </tr><tr>
                <td class="windowbg2">$uttxt{'6'}</td>
                <td class="windowbg2"><input type="checkbox" name="show_guest_today"${ischecked($show_guest_today)} /></td>
            </tr><tr>
                <td class="windowbg2">$uttxt{'11'}</td>
                <td class="windowbg2"><input type="checkbox" name="view_ips"${ischecked($view_ips)} /></td>
            </tr>~;
    }
    if ( $guest_ip == 1 && $log_only == 0 ) {
        $yymain .= qq~<tr>
                <td class="windowbg2">$uttxt{'10'}</td>
                <td class="windowbg2"><input type="checkbox" name="link_guest"${ischecked($link_guest)} /></td>
            </tr>~;
    }
    $yymain .= qq~
        </table>
    </div>
    <div class="bordercolor rightboxdiv">
        <table class="border-space pad-cell" style="margin-bottom: .5em;">
            <colgroup>
                <col style="width:50%" span="2" />
            </colgroup>
            <tr>
                <td class="titlebg" colspan="2">$admin_img{'prefimg'} <b>$uttxt{'41'}</b></td>
            </tr>~;
    if ( $guest_ip || $show_utlog || $show_max_stats ) {
        $yymain .= qq~            <tr>
                <td class="windowbg2">$uttxt{'34'}</td>
                <td class="windowbg2">
                    <select name="datestyle" size="1">
                        <option value="0"${isselected(!$datestyle || $datestyle == 0)}>$uttxt{'30'}</option>
                        <option value="1"${isselected($datestyle && $datestyle == 1)}>$uttxt{'31'}</option>
                        <option value="2"${isselected($datestyle && $datestyle == 2)}>$uttxt{'32'}</option>
                        <option value="3"${isselected($datestyle && $datestyle == 3)}>$uttxt{'33'}</option>
                    </select>
                </td>
            </tr>~;
    }
    $yymain .= qq~<tr>
                <td class="windowbg2">$uttxt{'9'}</td>
                <td class="windowbg2"><input type="checkbox" name="guest_ip"${ischecked($guest_ip)} /></td>
            </tr><tr>
                <td class="windowbg2">$uttxt{'9a'}</td>
                <td class="windowbg2"><input type="checkbox" name="stats"${ischecked($show_utlog)} /></td>
            </tr><tr>
                <td class="windowbg2">$uttxt{'9b'}</td>
                <td class="windowbg2"><input type="checkbox" name="max_stats"${ischecked($show_max_stats)} /></td>
            </tr>
        </table>
    </div>
    <div class="bordercolor rightboxdiv">
        <table class="border-space pad-cell" style="margin-bottom: .5em;">
            <colgroup>
                <col style="width:50%" span="2" />
            </colgroup>
            <tr>
                <td class="titlebg" colspan="2">$admin_img{'prefimg'} <b>$uttxt{'42'}</b></td>
            </tr>~;
    ## Delete old files after recovery ##
    if (   -e "$vardir/Mods/utoday/uttotal-old.txt"
        || -e "$vardir/Mods/utoday/utstats-old.txt" )
    {
        $yymain .= qq~<tr>
                <td class="windowbg2">$uttxt{'71'}</td>
                <td class="windowbg2"><input type="checkbox" name="delete_old"${ischecked($delete_old)} /></td>
            </tr>~;
    }

    ## Log Recovery Option ##
    if ( -e "$vardir/Mods/utoday/ut_clean_log_date.txt" ) {
        our ($UTCLD);
        fopen( 'UTCLD', '<', "$vardir/Mods/utoday/ut_clean_log_date.txt" );
        my @log_dates = <$UTCLD>;
        fclose('UTCLD');
        my $clean_log_files = @log_dates;
        chomp @log_dates;
        my $utcld         = $clean_log_files - 1;
        my $last_log_date = $log_dates[$utcld];
        my ( $utcldyear, $utcldmo, $utcldday ) = split /\-/xsm,
          $log_dates[$utcld];

        for my $checkday ( 1 .. 9 ) {
            my $checkday2 = "0$checkday";
            if ( $utcldmo eq $checkday2 )  { $utcldmo  = $checkday; }
            if ( $utcldday eq $checkday2 ) { $utcldday = $checkday; }
        }
        my $rec_log_date = qq~$utcldmo/$utcldday/$utcldyear~;
        if ( !$date_style || $date_style == 0 ) {
            $rec_log_date = qq~$utcldmo/$utcldday/$utcldyear~;
        }
        elsif ( $date_style == 1 ) {
            $utcldyear    = $utcldyear - 2000;
            $rec_log_date = qq~$utcldmo/$utcldday/0$utcldyear~;
        }
        elsif ( $date_style == 2 ) {
            $rec_log_date = qq~$utcldday.$utcldmo.$utcldyear~;
        }
        elsif ( $date_style == 3 ) {
            $utcldyear    = $utcldyear - 2000;
            $rec_log_date = qq~$utcldday.$utcldmo.0$utcldyear~;
        }
        $yymain .= qq~<tr>
                <td class="windowbg2">$uttxt{'64'}</td>
                <td class="windowbg2"><input type="checkbox" name="recover_log"${ischecked($recover_log)} /></td>
            </tr><tr>
                <td class="windowbg2" colspan="2"><span style="display: block; font-style: italic; padding: 0px 0px 5px 5px;">$uttxt{'65'}$rec_log_date.&nbsp;&nbsp;$uttxt{'66'}<b>$last_log_date</b>$uttxt{'69'}$uttxt{'68'}$uttxt{'67'}<b>$last_log_date</b>$uttxt{'69'}</span></td>
            </tr>~;
    }
    ## Clean Log Option ##
    if ( $fm && $fm != 0 ) {
        $yymain .= qq~<tr>
                <td class="windowbg2">$uttxt{'60'}</td>
                <td class="windowbg2"><input type="checkbox" name="clean_log"${ischecked($clean_log)} /></td>
            </tr>~;
    }
    ## Public Log Option ##
    $yymain .= qq~<tr>
                <td class="windowbg2">$uttxt{'59'}</td>
                <td class="windowbg2"><input type="checkbox" name="plogopt"${ischecked($plogopt)} /></td>
            </tr>~;
    if ( $plogopt && $plogopt == 1 ) {
        $yymain .= qq~<tr>
                <td class="windowbg2">$uttxt{'54'}</td>
                <td class="windowbg2"><select name="plogopt2" size="1"><option value="0"${isselected(!$plogopt2 || $plogopt2 == 0)}>$uttxt{'55'}</option><option value="1"${isselected($plogopt2 && $plogopt2 == 1)}>$uttxt{'56'}</option><option value="2"${isselected($plogopt2 && $plogopt2 == 2)}>$uttxt{'57'}</option><option value="3"${isselected($plogopt2 && $plogopt2 == 3)}>$uttxt{'58'}</option></select></td>
            </tr>~;
    }
    ## Admin Log Option ##
    $yymain .= qq~<tr>
                <td class="windowbg2">$uttxt{'50'}</td>
                <td class="windowbg2"><select name="logopt" size="1"><option value="0"${isselected(!$logopt || $logopt== 0)}>$uttxt{'51'}</option><option value="1"${isselected($logopt && $logopt == 1)}>$uttxt{'52'}</option><option value="2"${isselected($logopt && $logopt == 2)}>$uttxt{'53'}</option></select></td>
            </tr>~;
    if ( !$logopt || $logopt == 0 ) { $days_show = 0; }
    elsif ( $logopt && $logopt == 1 ) {
        if ( $fm && $fm != 0 ) {
            $yymain .= qq~<tr>
                <td class="windowbg2">$uttxt{'9c'}</td>
                <td class="windowbg2"><input type="text" name="days_show" size="5" value="$days_show" /></td>
            </tr>~;
        }
        else {
            $yymain .= qq~<tr>
                <td class="windowbg2" colspan="2"><span style="display: block; color: red; font-weight: bold; font-size: 12px; padding-top: 20px; padding: 20px 5px 0px 5px;">$uttxt{'29a'}</span></td>
            </tr>~;
        }
    }
    elsif ( $logopt == 2 ) {
        my $df               = q{};
        my $date_boxes_start = q{};
        my $date_boxes_end   = q{};
        if ($fm) {
            $sy2 ||= $fy;
            $ey2 ||= $ly;
            $sm  ||= $fm;
            $em  ||= $lm;
            $sd  ||= $fd;
            $ed  ||= $ld;
            $sy2 = sprintf '%02d', $sy2;
            $ey2 = sprintf '%02d', $ey2;

            if ( !$date_style || $date_style == 0 ) {
                $df = $uttxt{'30'};
                $date_boxes_start =
qq~<input type="text" name="sm" size="2" value="$sm" />/<input type="text" name="sd" size="2" value="$sd" />/<input type="text" name="sy4" size="4" value="$sy4" />~;
                $date_boxes_end =
qq~<input type="text" name="em" size="2" value="$em" />/<input type="text" name="ed" size="2" value="$ed" />/<input type="text" name="ey4" size="4" value="$ey4" />~;
            }
            elsif ( $date_style == 1 ) {
                $df = $uttxt{'31'};
                $date_boxes_start =
qq~<input type="text" name="sm" size="2" value="$sm" />/<input type="text" name="sd" size="2" value="$sd" />/<input type="text" name="sy2" size="2" value="$sy2" />~;
                $date_boxes_end =
qq~<input type="text" name="em" size="2" value="$em" />/<input type="text" name="ed" size="2" value="$ed" />/<input type="text" name="ey2" size="2" value="$ey2" />~;
            }
            elsif ( $date_style == 2 ) {
                $df = $uttxt{'32'};
                $date_boxes_start =
qq~<input type="text" name="sd" size="2" value="$sd" />.<input type="text" name="sm" size="2" value="$sm" />.<input type="text" name="sy4" size="4" value="$sy4" />~;
                $date_boxes_end =
qq~<input type="text" name="ed" size="2" value="$ed" />.<input type="text" name="em" size="2" value="$em" />.<input type="text" name="ey4" size="4" value="$ey4" />~;
            }
            elsif ( $date_style == 3 ) {
                $df = $uttxt{'33'};
                $date_boxes_start =
qq~<input type="text" name="sd" size="2" value="$sd" />.<input type="text" name="sm" size="2" value="$sm" />.<input type="text" name="sy2" size="2" value="$sy2" />~;
                $date_boxes_end =
qq~<input type="text" name="ed" size="2" value="$ed" />.<input type="text" name="em" size="2" value="$em" />.<input type="text" name="ey2" size="2" value="$ey2" />~;
            }
        }
        else {
            $date_boxes_start = q~<!--No log entries-->~;
            $date_boxes_end   = q~<!--No log entries-->~;
        }
        datestyle();
        if ( !$fm || $fm == 0 ) {
            $yymain .= qq~<tr>
                <td class="windowbg2" colspan="2"><span style="display: block; color: red; font-weight: bold; font-size: 12px; padding: 20px 5px 0px 5px;">$uttxt{'29'}</span></td>
            </tr>~;
        }
        else {
            my $date_range =
qq~$uttxt{'20'}$df$uttxt{'21'}<br/><br />$uttxt{'27'}<span style="color: red; font-weight: bold; text-decoration: underline;">$datelogopt21</span>$uttxt{'28'}<span style="color: red; font-weight: bold; text-decoration: underline;">$datelogopt22</span>~;
            $yymain .= qq~<tr>
                <td class="windowbg2">$date_range</td>
                <td class="windowbg2">
                    <table>
                        <tr>
                            <td style="width:100px">$uttxt{'22'}</td>
                            <td>$date_boxes_start</td>
                        </tr><tr>
                            <td>$uttxt{'23'}</td>
                            <td>$date_boxes_end</td>
                        </tr>
                    </table>
                </td>
            </tr>~;
        }

        my $badtxt = q{};
        if ( ( $baddates || $baddatee ) == 1 ) {
            if ( $baddates == 1 && $baddatee == 0 ) {
                $badtxt = qq~$uttxt{'24'}~;
            }
            elsif ( !$baddates && $baddatee == 1 ) {
                $badtxt = qq~$uttxt{'25'}~;
            }
            elsif ( $baddates && $baddatee == 1 ) {
                $badtxt = qq~$uttxt{'26'}~;
            }
            $yymain .= qq~<tr>
                <td class="titlebg" style="padding-left:1%" colspan="2"><span class="important">$badtxt</span></td>
            </tr>~;
        }
    }
    ## Recover Log Note ##
    if ( -e "$vardir/Mods/utoday/ut_clean_log_date.txt" ) {
        $yymain .= qq~<tr>
                <td class="windowbg2">$uttxt{'70'}</td>
                <td class="windowbg2"><input type="checkbox" name="log_only"${ischecked($log_only)} /></td>
            </tr>~;
    }
    if ( $clean_log == 1 ) {
        $yymain .= qq~<tr>
                <td class="titlebg" style="padding-left:1%" colspan="2"><b>$uttxt{'43'}</b></td>
            </tr><tr>
                <td class="windowbg2">$uttxt{'60a'}</td>
                <td class="windowbg2"><select name="clear_log_method" size="1"><option value="1"${isselected($clear_log_method && $clear_log_method == 1)}>$uttxt{'61'}</option><option value="2"${isselected($clear_log_method && $clear_log_method == 2)}>$uttxt{'62'}</option></select></td>
            </tr><tr>
                <td class="windowbg2">$uttxt{'63'}</td>
                <td class="windowbg2"><input type="checkbox" name="log_only"${ischecked($log_only)} /></td>
            </tr>~;
    }
    $yymain .= qq~
        </table>
    </div>
    <div class="bordercolor rightboxdiv">
        <table class="border-space pad-cell" style="margin-bottom: .5em;">
            <tr>
                <th class="titlebg">$admin_img{'prefimg'} $admin_txt{'10'}</th>
            </tr><tr>
                <td class="catbg center"><input type="submit" value="$admin_txt{'10'}" /></td>
            </tr>
        </table>
    </div>
</form>~;

    ## Set up divisions for Statistics and/or Guest IP List ##
    if ( ( $show_utlog || $show_max_stats ) && $guest_ip == 1 ) {
        my $getstats = statistics();
        my $gst_ip   = guest_ip_list();
        $yymain .= qq~
        <div class="clear"></div>
        <div style="padding: 0px; width: 99%; margin-left: 0px; margin-right: auto;">
            <div class="bordercolor" style="float: left; padding: 0px; width: 49%; margin-left: 0px; margin-right: auto;">
            $getstats
            </div>
            <div class="bordercolor" style="float: right; padding: 0px; width: 49%; margin-right: 0px; margin-left: auto;">
            $gst_ip
            </div>
        </div>~;

    }
    elsif ( ( $show_utlog || $show_max_stats ) && !$guest_ip ) {
        my $getstats = statistics();
        $yymain .= qq~
        <div class="bordercolor" style="padding: 0px; width: 50%; margin-left: auto; margin-right: auto;">
        $getstats
        </div>~;

    }
    elsif ( ( !$show_utlog || !$show_max_stats ) && $guest_ip ) {
        my $gst_ip = guest_ip_list();
        $yymain .= q~
        <div class="bordercolor" style="padding: 0px; width: 50%; margin-left: auto; margin-right: auto;">
        $gst_ip
        </div>~;
    }
    else {
        $yymain .= q~<!--Nothing selected -->~;
    }

    $yytitle     = $admin_txt{'222'};
    $action_area = 'userstoday';
    admintemplate();
    exit;
}

sub userstoday_set2 {
    is_admin_or_gmod();

    $log_only = $FORM{'log_only'} ? 1 : 0;
    $date_style = $FORM{'datestyle'} || 0;
    $guest_view       = $FORM{'guest_view'}       ? 1 : 0;
    $show_guest_today = $FORM{'show_guest_today'} ? 1 : 0;
    $view_ips         = $FORM{'view_ips'}         ? 1 : 0;
    $guest_ip         = $FORM{'guest_ip'}         ? 1 : 0;
    $show_utlog       = $FORM{'stats'}            ? 1 : 0;
    $show_max_stats   = $FORM{'max_stats'}        ? 1 : 0;
    $days_show = $FORM{'days_show'} || 0;
    $link_guest = $FORM{'link_guest'} ? 1 : 0;
    $logopt = $FORM{'logopt'} || 0;
    $plogopt = $FORM{'plogopt'} ? 1 : 0;
    $plogopt2 = $FORM{'plogopt2'} || 0;
    $recover_log = $FORM{'recover_log'} ? 1 : 0;
    $delete_old  = $FORM{'delete_old'}  ? 1 : 0;
    $clean_log   = $FORM{'clean_log'}   ? 1 : 0;
    $clear_log_method = $FORM{'clear_log_method'} || 0;
    $sm               = $FORM{'sm'}               || 0;
    $sd               = $FORM{'sd'}               || 0;
    $sy2              = $FORM{'sy2'}              || ( $FORM{'sy4'} - 2000 );
    $sy4              = $FORM{'sy4'}              || 0;
    $em               = $FORM{'em'}               || 0;
    $ed               = $FORM{'ed'}               || 0;
    $ey2              = $FORM{'ey2'}              || ( $FORM{'ey4'} - 2000 );
    $ey4              = $FORM{'ey4'}              || 0;

    recover_log();
    delete_old();
    check_dates();

    $sd =~ s/^0+//xsm;
    $sm =~ s/^0+//xsm;
    $ed =~ s/^0+//xsm;
    $em =~ s/^0+//xsm;
    $sy2 =~ s/^0+//xsm;
    $ey2 =~ s/^0+//xsm;
    $fm =~ s/^0+//xsm;
    $fd =~ s/^0+//xsm;
    $fy =~ s/^0+//xsm;
    $lm =~ s/^0+//xsm;
    $ld =~ s/^0+//xsm;
    $ly =~ s/^0+//xsm;
    clear_log_method();

    my $setfile = << "EOF";
###############################################################################
# usrtodayconfig.txt (Users Today configurations)                             #
###############################################################################

\$log_only = '$log_only'; # set to 1 to only use the log function in the Admin Center and not show Users Today block in Board Index
\$date_style = '$date_style'; # set individual users' date style preference
\$guest_view = '$guest_view'; # set to 1 to allow guests to see the Guests/Members Online Today list
\$show_guest_today = '$show_guest_today'; # set to 1 to show number of guests in Online Today list
\$user_today_hidden_show = '$user_today_hidden_show'; # set to 1 to keep hidden users off of the Users Today listing
\$view_ips = '$view_ips'; # set to 1 to view guest IP addresses in the Users Today block
\$show_utlog = '$show_utlog'; # set to 1 to see the Users Today Statistical Log in the Users Today Administration area
\$show_max_stats = '$show_max_stats'; #set to 1 to see the most, total and average user/guest stats
\$guest_ip = '$guest_ip'; # set to 1 to view the Guest IP Log in the Users Today Administration area
\$days_show = '$days_show'; # Number of days to show log
\$link_guest = '$link_guest'; # set to 1 to link the number of guests who've visited the board to view the Guest IP Log in the Users Today Administration area
\$logopt = '$logopt'; # determines how the log will be viewed - either full, X days from today, or between two dates
\$plogopt = '$plogopt'; # set to 1 to enable the public log
\$plogopt2 = '$plogopt2'; # determines who is allowed to view the public log
\$clean_log = '$clean_log'; # set to 1 to clean the statistical logs
\$clear_log_method = '$clear_log_method'; # if value equals 2, log will be cleaned on a yearly basis
\$sm = '$sm'; # starting log month
\$sd = '$sd'; # starting log day
\$sy2 = '$sy2'; # starting log year (2 digit)
\$sy4 = '$sy4'; # starting log year (4 digit)
\$em = '$em'; # ending log month
\$ed = '$ed'; # ending log day
\$ey2 = '$ey2'; # ending log year (2 digit)
\$ey4 = '$ey4'; # ending log year (4 digit)
\$baddates = '$baddates'; # if at 1, indicates a bad starting date was entered
\$baddatee = '$baddatee'; # if at 1, indicates a bad ending date was entered
\$run = '$run'; # just a counter
\$fm = '$fm';
\$fd = '$fd';
\$fy = '$fy';
\$lm = '$lm';
\$ld = '$ld';
\$ly = '$ly';

1;
EOF

    if ( !-d 'Variables/Mods/utoday' ) {
        my $newdir = 'Variables/Mods/utoday';
        mkdir $newdir, 0755;
    }
    our ($FILE);
    fopen( 'FILE', '>', "$vardir/Mods/utoday/usrtodayconfig.txt" );
    print {$FILE} $setfile or croak "$croak{'print'} FILE";
    fclose('FILE');

    if ( $action eq 'userstoday2' ) {
        $yysetlocation = qq~$adminurl?action=userstoday~;
        redirectexit();
    }
    return;
}

## Create Guest IP List ##
sub guest_ip_list {
    my $return = q{};
    my $gst_ip = q{};
    if ( -e "$vardir/Mods/utoday/guesttoday.txt" ) {
        our ($GUESTFILE);
        fopen( 'GUESTFILE', '<', "$vardir/Mods/utoday/guesttoday.txt" );
        my @gtoday = <$GUESTFILE>;
        fclose('GUESTFILE');
        chomp @gtoday;
        my $guesttodaycount = @gtoday;
        for my $g ( 1 .. $#gtoday ) {
            $gtoday[$g] =~ s/\n//igxsm;
            my $g_ip = $gtoday[$g];
            $gst_ip .= qq~
            <span style="padding: 0px 5px; font-size: 12px;">$g_ip</span><br />~;
        }
        $guesttodaycount = $guesttodaycount - 1;
        $gday            = $gtoday[0];
        datestyle();
        $return = << "GST";
        <table class="border-space pad-cell">
            <tr>
                <th class="titlebg center"><b>$uttxt{'7'}</b><span style="color: black; text-shadow:0 0 0 #4378ae">$dateformat_guest</span></th>
            </tr><tr>
                <td class="windowbg2">
                    <div style="width: 100%; overflow: auto; height: 200px; padding: 2px 0px 2px 2px; margin: -1px 0px -1px -1px;">
$gst_ip
                    </div>
                </td>
            </tr><tr>
                <td class="catbg">$uttxt{'8'}<span class="important">$guesttodaycount</span></td>
            </tr>
        </table>
GST
    }
    return $return;
}
## End Guest IP List ##

## Create Statistics Table ##
sub statistics {
    ## Show "Max Stats" table ##
    my $statstab = q~
        <table class="border-space pad-cell">
            <colgroup>
                <col style="width:40%" />
                <col style="width:30%" span="2" />
            </colgroup>~;
    if ( $show_max_stats == 1 ) {
        if ( -e "$vardir/Mods/utoday/utstats.txt" ) {
            our ($UTSTATS);
            fopen( 'UTSTATS', '<', "$vardir/Mods/utoday/utstats.txt" );
            my @utstats = <$UTSTATS>;
            fclose('UTSTATS');
            ( $utmostuser, $utmumo, $utmuday, $utmuyr ) = split /[|]/xsm,
              $utstats[0] || q{};
            ( $utmostguest, $utmgmo, $utmgday, $utmgyr ) = split /[|]/xsm,
              $utstats[1] || q{};
            my $uttotaluser  = $utstats[2];
            my $uttotalguest = $utstats[3];
            my $utlogdays    = $utstats[4];
            my $utavguser    = $utstats[5];
            my $utavgguest   = $utstats[6];
            datestyle();

            my $mostuser = q~<!--No Data-->~;
            if ($utmumo) {
                $utmostuser            ||= q{};
                $dateformat_user_stats ||= q{};
                $mostuser =
qq~<b>$utmostuser</b><br /><span style="color: red; font-style: italic;">($dateformat_user_stats)</span>~;
            }
            my $mostguest = q~<!--No Data-->~;
            if ($utmgmo) {
                $utmostguest            ||= q{};
                $dateformat_guest_stats ||= q{};
                $mostguest =
qq~<b>$utmostguest</b><br /><span style="color: red; font-style: italic;">($dateformat_guest_stats)</span>~;
            }
            $statstab .= qq~
            <tr>
                <td class="titlebg center" colspan="3"><b>$uttxt{'17'}</b></td>
            </tr><tr>
                <td class="catbg">&nbsp;</td>
                <td class="catbg center"><b>$uttxt{'14'}</b></td>
                <td class="catbg center"><b>$uttxt{'15'}</b></td>
            </tr><tr>
                <td class="windowbg2"><b>$uttxt{'18'}</b></td>
                <td class="windowbg2 center">$mostuser</td>
                <td class="windowbg2 center">$mostguest</td>
            </tr><tr>
                <td class="windowbg2">$uttxt{'19b'}</td>
                <td class="windowbg2 center">$uttotaluser</td>
                <td class="windowbg2 center">$uttotalguest</td>
            </tr><tr>
                <td class="windowbg2">$uttxt{'19a'}</td>
                <td class="windowbg2 center">$utavguser</td>
                <td class="windowbg2 center">$utavgguest</td>
            </tr><tr>
                <td class="windowbg2">$uttxt{'19c'}</td>
                <td colspan="2" class="windowbg2 center">$utlogdays</td>
            </tr>~;
        }
    }
    if ( $show_utlog == 1 ) {
        $statstab .= qq~<tr>
                <td class="titlebg center" colspan="3"><b>$uttxt{'16'}</b></td>
            </tr><tr>
                <td class="windowbg2"><b>$uttxt{'13'}</b></td>
                <td class="windowbg2 center"><b>$uttxt{'14'}&nbsp;$uttxt{'14a'}</b></td>
                <td class="windowbg2 center"><b>$uttxt{'15'}</b></td>
            </tr>~;
        if ( -e "$vardir/Mods/utoday/uttotal.txt" ) {
            our ($TOTUSER);
            fopen( 'TOTUSER', '<', "$vardir/Mods/utoday/uttotal.txt" );
            my @totusers = <$TOTUSER>;
            fclose('TOTUSER');
            my $countlog = @totusers;
            my $ut       = 0;
            if ( $logopt == 2 ) {
                $days_show = 0;
                my $start_pos  = -1;
                my $finish_pos = -1;
                $sm ||= 0;
                $sd ||= 0;
                $em ||= 0;
                $ed ||= 0;
                $sm = sprintf '%02d', $sm;
                $sd = sprintf '%02d', $sd;
                $em = sprintf '%02d', $em;
                $ed = sprintf '%02d', $ed;

                my $endoflog = $count_log - 1;
                ( $finish_pos, $start_pos ) = get_pos(@totusers);

                if ( ( $start_pos && $finish_pos ) > -1 ) {
                    $ut       = $start_pos;
                    $countlog = $finish_pos + 1;
                }
                elsif ( $start_pos > -1 )  { $ut       = $start_pos; }
                elsif ( $finish_pos > -1 ) { $countlog = $finish_pos + 1; }
                else                       { $ut = $countlog - $days_show; }
            }
            else { $ut = $countlog - $days_show; }
            if ( $ut eq $countlog || $ut < 0 ) { $ut = 0; }
            for my $t ( $ut .. ( $countlog - 1 ) ) {
                (
                    $utmonth, $utday, $utyear, $utdow, $totuser, $mempercent,
                    $totguest
                ) = split /[|]/xsm, $totusers[$t];
                datestyle();
                $statstab .= qq~<tr>
                <td class="windowbg2">$dateformat_log</td>
                <td class="windowbg2 center">$totuser&nbsp;<span style="color: red; font-style: italic;">($mempercent)</span></td>
                <td class="windowbg2 center">$totguest</td>
            </tr>~;
            }
        }
    }
    $statstab .= q~
     </table>~;
    return $statstab;
}

## End Statistics Table ##

## define $dateformat ##
sub datestyle {
    my $iplogyear = $year - 2000;
    my ( $sec, $min, $hour, $mday, $mon, $yr, $wday, $yday, undef ) = gmtime;
    $gday ||= $mday;
	$yr += 1900;
    my $mon_num = $mon + 1;
    $utmonth ||= $mon_num;
    $utmumo  ||= $mon_num;
    $utmgmo  ||= $mon_num;
    $utday   ||= $mday;
    $utmuday ||= $mday;
    $utmgday ||= $mday;
    $utyear  ||= ( $yr - 100 );
    $utmuyr  ||= ( $yr - 100 );
    $utmgyr  ||= ( $yr - 100 );
    if ( !$date_style || $date_style == 0 ) {
        $fy ||= 0;
        $ly ||= 0;
        $fy                     = $fy + 2000;
        $ly                     = $ly + 2000;
        $dateformat_guest       = qq~, $mon_num/$gday/$yr~;
        $dateformat_log         = qq~$utmonth/$utday/20$utyear~;
        $dateformat_user_stats  = qq~$utmumo/$utmuday/20$utmuyr~;
        $dateformat_guest_stats = qq~$utmgmo/$utmgday/20$utmgyr~;
        $datelogopt21           = qq~$fm/$fd/$fy~;
        $datelogopt22           = qq~$lm/$ld/$ly~;
    }
    elsif ( $date_style == 1 ) {
        $iplogyear              = sprintf '%02d', $iplogyear;
        $dateformat_guest       = qq~, $mon_num/$gday/$iplogyear~;
        $dateformat_log         = qq~$utmonth/$utday/$utyear~;
        $dateformat_user_stats  = qq~$utmumo/$utmuday/$utmuyr~;
        $dateformat_guest_stats = qq~$utmgmo/$utmgday/$utmgyr~;
        $datelogopt21           = qq~$fm/$fd/0$fy~;
        $datelogopt22           = qq~$lm/$ld/0$ly~;
    }
    elsif ( $date_style == 2 ) {
        $fy                     = $fy + 2000;
        $ly                     = $ly + 2000;
        $dateformat_guest       = qq~, $gday.$mon_num.$yr~;
        $dateformat_log         = qq~$utday.$utmonth.20$utyear~;
        $dateformat_user_stats  = qq~$utmuday.$utmumo.20$utmuyr~;
        $dateformat_guest_stats = qq~$utmgday.$utmgmo.20$utmgyr~;
        $datelogopt21           = qq~$fd.$fm.$fy~;
        $datelogopt22           = qq~$ld.$lm.$ly~;
    }
    elsif ( $date_style == 3 ) {
        $iplogyear              = sprintf '%02d', $iplogyear;
        $dateformat_guest       = qq~, $gday.$mon_num.$iplogyear~;
        $dateformat_log         = qq~$utday.$utmonth.$utyear~;
        $dateformat_user_stats  = qq~$utmuday.$utmumo.$utmuyr~;
        $dateformat_guest_stats = qq~$utmgday.$utmgmo.$utmgyr~;
        $datelogopt21           = qq~$fd.$fm.0$fy~;
        $datelogopt22           = qq~$ld.$lm.0$ly~;
    }
    return;
}

sub check_dates {
    if ( ( $fm && $fm < 1 ) || ( $recover_log && $recover_log == 1 ) ) {
        if ( -e "$vardir/Mods/utoday/uttotal.txt" ) {
            our ($UTLOG);
            fopen( 'UTLOG', '<', "$vardir/Mods/utoday/uttotal.txt" );
            my @utentries = <$UTLOG>;
            fclose('UTLOG');
            my $utlines     = @utentries;
            my $skip        = 0;
            my $start_good  = -1;
            my $finish_good = -1;
            $sm = sprintf '%02d', $sm;
            $sd = sprintf '%02d', $sd;
            $em = sprintf '%02d', $em;
            $ed = sprintf '%02d', $ed;
            my $endoflog = $count_log - 1;
            ( $logms, $logds, $logys, undef, undef, undef, undef ) =
              split /[|]/xsm, $utentries[0];
            ( $logme, $logde, $logye, undef, undef, undef, undef ) =
              split /[|]/xsm, $utentries[ $utlines - 1 ];

            while ( $skip < $utlines ) {
                my ( $logmonth, $logday, $logyear, undef, undef, undef, undef )
                  = split /[|]/xsm, $utentries[$skip];
                if ( $date_style == 0 || $date_style == 2 ) {
                    $logyear = "20$logyear";
                    if (   ( $logmonth eq $sm )
                        && ( $logday eq $sd )
                        && ( $logyear eq $sy4 ) )
                    {
                        $start_good = $skip;
                    }
                    if (   ( $logmonth eq $em )
                        && ( $logday eq $ed )
                        && ( $logyear eq $ey4 ) )
                    {
                        $finish_good = $skip;
                    }
                }
                else {
                    if (   ( $logmonth eq $sm )
                        && ( $logday eq $sd )
                        && ( $logyear eq $sy2 ) )
                    {
                        $start_good = $skip;
                    }
                    if (   ( $logmonth eq $em )
                        && ( $logday eq $ed )
                        && ( $logyear eq $ey2 ) )
                    {
                        $finish_good = $skip;
                    }
                }
                $skip++;
            }
            if ( $logopt == 2 ) {
                if   ( $start_good > -1 ) { $baddates = 0; }
                else                      { $baddates = 1; set_start_date(); }
                if   ( $finish_good > -1 ) { $baddatee = 0; }
                else                       { $baddatee = 1; set_end_date(); }
            }
            if ( $logopt != 2 ) {
                $baddates = 0;
                $baddatee = 0;
                set_start_date();
                set_end_date();
                $run = -1;
            }
            if ( $logopt == 2 ) { $run++; }
            if ( $run < 1 ) { $baddates = 0; $baddatee = 0; }
            $fm = $logms;
            $fd = $logds;
            $fy = $logys;
            $lm = $logme;
            $ld = $logde;
            $ly = $logye;
        }
    }
    return;
}

sub set_start_date {
    $sm  = $logms;
    $sd  = $logds;
    $sy4 = 2000 + $logys;
    $sy2 = $logys;
    return;
}

sub set_end_date {
    $em  = $logme;
    $ed  = $logde;
    $ey4 = 2000 + $logye;
    $ey2 = $logye;
    return;
}

sub clear_log_method {
    if ( $fm && $fm != 0 ) {
        if ( $clean_log && $clean_log == 1 ) {
            our ($UTCLD);
            if ( !-e "$vardir/Mods/utoday/ut_clean_log_date.txt" ) {
                fopen( 'UTCLD', '>',
                    "$vardir/Mods/utoday/ut_clean_log_date.txt" );
                fclose('UTCLD');
            }
            if ( $clear_log_method == 1 ) {
                $ly = $ly + 2000;
                $lm = sprintf '%02d', $lm;
                $ld = sprintf '%02d', $ld;
                fopen( 'UTCLD', '>>',
                    "$vardir/Mods/utoday/ut_clean_log_date.txt" );
                print {$UTCLD} "$ly-$lm-$ld\n" or croak "$croak{'print'} UTCLD";
                fclose('UTCLD');
                rename "$vardir/Mods/utoday/uttotal.txt",
                  "$vardir/Mods/utoday/uttotal-$ly-$lm-$ld.txt";
                rename "$vardir/Mods/utoday/utstats.txt",
                  "$vardir/Mods/utoday/utstats-$ly-$lm-$ld.txt";
                fopen( 'UTTOTAL', '>', "$vardir/Mods/utoday/uttotal.txt" );
                fclose('UTTOTAL');
                fopen( 'UTSTATS', '>', "$vardir/Mods/utoday/utstats.txt" );
                fclose('UTSTATS');
                $sm               = 0;
                $sd               = 0;
                $sy2              = 0;
                $sy4              = 0;
                $em               = 0;
                $ed               = 0;
                $ey2              = 0;
                $ey4              = 0;
                $fm               = 0;
                $fd               = 0;
                $fy               = 0;
                $lm               = 0;
                $ld               = 0;
                $ly               = 0;
                $baddates         = 0;
                $baddatee         = 0;
                $run              = -1;
                $clean_log        = 0;
                $clear_log_method = 0;
            }
            if ( $clear_log_method == 2 ) {
                if ( $lm == 12 && $ld == 31 ) {
                    our ($YTCLD);
                    fopen( 'UTCLD', '>>',
                        "$vardir/Mods/utoday/ut_clean_log_date.txt" );
                    $ly = $ly + 2000;
                    $lm = sprintf '%02d', $lm;
                    $ld = sprintf '%02d', $ld;
                    print {$UTCLD} "$ly-$lm-$ld\n"
                      or croak "$croak{'print'} UTCLD";
                    fclose('UTCLD');
                    rename "$vardir/Mods/utoday/uttotal.txt",
                      "$vardir/Mods/utoday/uttotal-$ly-$lm-$ld.txt";
                    rename "$vardir/Mods/utoday/utstats.txt",
                      "$vardir/Mods/utoday/utstats-$ly-$lm-$ld.txt";
                    fopen( 'UTTOTAL', '>', "$vardir/Mods/utoday/uttotal.txt" );
                    fclose('UTTOTAL');
                    fopen( 'UTSTATS', '>', "$vardir/Mods/utoday/utstats.txt" );
                    fclose('UTSTATS');
                    $sm       = 0;
                    $sd       = 0;
                    $sy2      = 0;
                    $sy4      = 0;
                    $em       = 0;
                    $ed       = 0;
                    $ey2      = 0;
                    $ey4      = 0;
                    $fm       = 0;
                    $fd       = 0;
                    $fy       = 0;
                    $lm       = 0;
                    $ld       = 0;
                    $ly       = 0;
                    $baddates = 0;
                    $baddatee = 0;
                    $run      = -1;
                }
            }
        }
    }
    return;
}

sub recover_log {
    if ( $recover_log == 1 ) {
        if ( -e "$vardir/Mods/utoday/ut_clean_log_date.txt" ) {
            our ($UTCLD);
            fopen( 'UTCLD', '<', "$vardir/Mods/utoday/ut_clean_log_date.txt" );
            my @log_dates = <$UTCLD>;
            fclose('UTCLD');
            my $clean_log_files = @log_dates;
            chomp @log_dates;
            my $utcld         = $clean_log_files - 1;
            my $last_log_date = $log_dates[$utcld];
            rename "$vardir/Mods/utoday/uttotal.txt",
              "$vardir/Mods/utoday/uttotal-old.txt";
            rename "$vardir/Mods/utoday/utstats.txt",
              "$vardir/Mods/utoday/utstats-old.txt";
            rename "$vardir/Mods/utoday/uttotal-$last_log_date.txt",
              "$vardir/Mods/utoday/uttotal.txt";
            rename "$vardir/Mods/utoday/utstats-$last_log_date.txt",
              "$vardir/Mods/utoday/utstats.txt";
            unlink "$vardir/Mods/utoday/ut_clean_log_date.txt";
        }
    }
    return;
}

sub delete_old {
    if ( $delete_old == 1 ) {
        unlink "$vardir/Mods/utoday/uttotal-old.txt";
        unlink "$vardir/Mods/utoday/utstats-old.txt";
    }
    return;
}

sub get_pos {
    my @totusers = @_;
    my $countlog = @totusers;
    my ( $finish_pos, $start_pos );
    $count_log = 0;
    while ( $count_log < $countlog ) {
        my ( $logmonth, $logday, $logyear, undef, undef, undef, undef ) =
          split /[|]/xsm, $totusers[$count_log];
        if ( !$date_style || $date_style == 0 || $date_style == 2 ) {
            $logyear = "20$logyear";
            if (   $logmonth eq $sm
                && $logday eq $sd
                && $logyear eq $sy4 )
            {
                $start_pos = $count_log;
            }
            if (   $logmonth eq $em
                && $logday eq $ed
                && $logyear eq $ey4 )
            {
                $finish_pos = $count_log;
            }
        }
        else {
            if (   $logmonth eq $sm
                && $logday eq $sd
                && $logyear eq $sy2 )
            {
                $start_pos = $count_log;
            }
            if (   $logmonth eq $em
                && $logday eq $ed
                && $logyear eq $ey2 )
            {
                $finish_pos = $count_log;
            }
        }
        $count_log++;
    }
    return ( $finish_pos, $start_pos );
}

1;
